import { Stack, Tabs } from '@mantine/core'
import useSWR from 'swr'
import { BASE_URL } from '../constants'
import { fetcher } from '../http/axiosInstance'
import { useState } from 'react'
import CustomTable from '../components/CustomTable'

const DBManager = () => {
    const { data: tablesData } = useSWR(`/db/tables`, (key) => fetcher(key, BASE_URL.ems), {
        revalidateOnFocus: false
    })

    return (
        <Stack w={'100%'} h={'100%'} p='xs'>
            {tablesData && Array.isArray(tablesData) && tablesData.length > 0 &&
                <Tabs w='100%' h='80%'>
                    <Stack h='100%'>
                        <Tabs.List>
                            {tablesData.map(table => (
                                <Tabs.Tab key={table.tablename} value={table.tablename}>
                                    {table.tablename}
                                </Tabs.Tab>
                            ))}
                        </Tabs.List>

                        {tablesData.map(table => (
                            <Tabs.Panel h='100%' key={table.tablename} value={table.tablename} w='100%'>
                                <TableData tablename={table.tablename} />
                            </Tabs.Panel>
                        ))}
                    </Stack>

                </Tabs>
            }
            {/* <Card withBorder radius='sm'>
                <Stack>
                    <Group justify='flex-start'>
                        <Text fw='bold'>Figures</Text>
                    </Group>
                    <Grid>
                        <Button>Import from New_Gis (erases data)</Button>
                    </Grid>
                </Stack>
            </Card> */}
        </Stack>
    )
}

const TableData = ({
    tablename
}: { tablename: string }) => {
    const [offset] = useState(0)
    const [limit] = useState(20)

    const { data: columnsData } = useSWR(`/db/columns/${tablename}`, (key) => fetcher(key, BASE_URL.ems), {
        revalidateOnFocus: false
    })

    const { data: rowsData } = useSWR(columnsData ? `/db/rows/${tablename}?offset=${offset}&limit=${limit}` : null, (key) => fetcher(key, BASE_URL.ems), {
        revalidateOnFocus: false
    })

    return (
        <>
            {columnsData && rowsData && Array.isArray(columnsData) && Array.isArray(rowsData) && columnsData.length > 0 &&
                <CustomTable data={rowsData} columns={columnsData.map(column => (
                    {
                        accessorKey: column.column_name,
                        header: column.column_name,
                        cell: (info) => JSON.stringify(info.getValue()).length > 30 ? [JSON.stringify(info.getValue()).substring(0, 30), '...'].join('') : JSON.stringify(info.getValue()),
                    }
                ))} />
            }
        </>
    )
}

export default DBManager